@@ -86,8 +86,14 @@ class Raster(object):
                             'bottom':self.extent[2], 'top':self.extent[3]}
         
         # raster value array
-        self.array = array
-        self.array[array == header['NODATA_value']] = np.nan
+        
+        ind = array == header['NODATA_value']
+        if ind.sum() > 0:
+            self.array = array+0.0
+            self.array[ind] = np.nan
+        else:
+            self.array = array
+            
         
         # shape and cellsize
         self.shape = self.array.shape
@@ -101,20 +107,32 @@ class Raster(object):
         self.size = self.array.size
         
         # summary
-        self.summary = copy.deepcopy(self.header)
-        self.summary['num_valid_cells'] = self.num_valid_cells
-        if hasattr(self, 'crs'):
-            self.summary['crs'] = crs.data
-        if hasattr(self, 'source_file'):
-            self.summary['source_file'] = source_file
+        self.get_summary()
+        
         self.to_rasterio = self.write_tif
 
             
 #%%============================= Spatial analyst ==============================   
+    def get_summary(self):
+        """Get information summary of the object
+
+        Returns
+        -------
+        summary : dict
+            information summary of the object.
+
+        """
+        summary = copy.deepcopy(self.header)
+        summary['num_valid_cells'] = self.num_valid_cells
+        if hasattr(self, 'crs'):
+            summary['crs'] = self.crs.data
+        if hasattr(self, 'source_file'):
+            summary['source_file'] = self.source_file
+        self._summary = summary
+        return summary
+
     def set_crs(self, crs):
         """set reference coordinate system
-        
-
         Parameters
         ----------
         crs : int|string|rasterio.crs object
@@ -145,14 +163,32 @@ class Raster(object):
         xllcorner = min(x_centre)-0.5*self.header['cellsize']
         yllcorner = min(y_centre)-0.5*self.header['cellsize']
         header_new = copy.deepcopy(self.header)
-        array_new = self.array[min(rows):max(rows), min(cols):max(cols)]
+        if max(rows) >= self.header['nrows']:
+            max_row = self.header['nrows']
+        else:
+            max_row = max(rows)+1
+        if max(cols) >= self.header['ncols']:
+            max_col = self.header['ncols']
+        else:
+            max_col = max(cols)+1
+        if min(rows) <= 0:
+            min_row = 0
+        else:
+            min_row = min(rows) 
+        if min(cols) <= 0:
+            min_col = 0
+        else:
+            min_col = min(cols) 
+        array_new = self.array[min_row:max_row,
+                               min_col:max_col]
         header_new['nrows'] = array_new.shape[0]
         header_new['ncols'] = array_new.shape[1]
         header_new['xllcorner'] = xllcorner
         header_new['yllcorner'] = yllcorner
-        new_obj = Raster(array=array_new, header=header_new,
-                         wkt=self.wkt)
-        return new_obj
+        obj_new = Raster(array=array_new, header=header_new)
+        if hasattr(self, 'crs'):
+            obj_new.crs = self.crs
+        return obj_new
     
     def clip(self, clip_mask=None):
         """clip raster according to a mask
@@ -175,15 +211,21 @@ class Raster(object):
         else:
             raise IOError('mask must be either a string or a numpy array')
         
-        out_image, out_transform = mask.mask(ds_rio, shapes, crop=True) # 
-        array = out_image[0]
+        out_image, out_transform = mask.mask(ds_rio, shapes, crop=True) #
+        ds_rio.close()
+        array_new = out_image[0]
         cellsize = out_transform[0]
         xllcorner = out_transform[2]
-        yllcorner = out_transform[5]- cellsize*array.shape[0]
-        obj_clipped = Raster(array=array, xllcorner=xllcorner,
-                             yllcorner=yllcorner, NODATA_value=ds_rio.nodata,
-                             cellsize=cellsize)
-        return obj_clipped
+        yllcorner = out_transform[5]- cellsize*array_new.shape[0]
+        header_new = copy.deepcopy(self.header)
+        header_new['nrows'] = array_new.shape[0]
+        header_new['ncols'] = array_new.shape[1]
+        header_new['xllcorner'] = xllcorner
+        header_new['yllcorner'] = yllcorner
+        obj_new = Raster(array=array_new, header=header_new)
+        if hasattr(self, 'crs'):
+            obj_new.crs = self.crs
+        return obj_new
     
     def rasterize(self, shp_filename):
         """
@@ -205,12 +247,13 @@ class Raster(object):
                 shapes = [feature['geometry'] for feature in shapefile]
         else:
             shapes = shp_filename
+        shapes = [x for x in shapes if x != None]
         out_image, _ = mask.mask(ds_rio, shapes) #, crop=True
         rasterized_array = out_image[0]
         rasterized_array[np.isnan(rasterized_array)] = ds_rio.nodata
         index_array = np.full(rasterized_array.shape, True)
         index_array[rasterized_array == ds_rio.nodata] = False
-#        ds_rio.close()
+        ds_rio.close()
         return index_array
         
     def resample(self, new_cellsize, method='bilinear'):
@@ -249,6 +292,8 @@ class Raster(object):
         new_header['xllcorner'] = transform[2]
         new_header['yllcorner'] = transform[5]-data.shape[0]*new_cellsize
         obj_new = Raster(array=data, header=new_header)
+        if hasattr(self, 'crs'):
+            obj_new.crs = self.crs
         return obj_new
         
     
@@ -270,10 +315,9 @@ class Raster(object):
         grid_x, grid_y = self.to_points()
         array_interp = interpolate.griddata(points, values, (grid_x, grid_y),
                                             method=method)
-        new_obj = copy.deepcopy(self)
-        new_obj.array = array_interp
-        new_obj.source_file = self.source_file
-        return new_obj
+        obj_new = copy.deepcopy(self)
+        obj_new.array = array_interp
+        return obj_new
     
     def grid_interpolate(self, value_grid, method='nearest'):
         """ Interpolate values of a grid to all cells on the Raster object
@@ -328,13 +372,15 @@ class Raster(object):
         array = self.array[rows, cols]
         array = array.transpose()
         array = array.astype(self.array.dtype)
-        new_obj = Raster(array=array, header=header)
-        return new_obj
+        obj_new = Raster(array=array, header=header)
+        if hasattr(self, 'crs'):
+            obj_new.crs = self.crs
+        return obj_new
     
     def assign_to(self, new_header):
         """ Assign_to the object to a new grid defined by new_header 
 
-        If cellsize are not equal, the origin Raster will be firstly 
+        If their cellsizes are not equal, the original Raster will be 
         resampled to the target grid.
 
         Return:
@@ -352,8 +398,35 @@ class Raster(object):
         new_array = self.array[rows, cols]
         new_array = new_array+0.0
         new_array[new_array == self.header['NODATA_value']] = np.nan
-        obj_output = Raster(array=new_array, header=new_header)
-        return obj_output
+        obj_new = Raster(array=new_array, header=new_header)
+        if hasattr(self, 'crs'):
+            obj_new.crs = self.crs
+        return obj_new
+    
+    def paste_on(self, obj_large):
+        """ Paste the object to a larger grid defined by obj_large and
+        replace corresponding grid values with the object array
+
+        If their cellsizes MUST be equal
+        """
+        header_s = self.header
+        header_l = obj_large.header
+        x, y = self.to_points()
+        r0, c0 = sp.map2sub(self.extent[0]+self.cellsize/2, 
+                            self.extent[-1]-self.cellsize/2,
+                            header_l)
+        rows = np.arange(r0, r0+header_s['nrows'])
+        cols = np.arange(c0, c0+header_s['ncols'])
+        # cut array outside the range of the large raster grid
+        ind_r = np.logical_and(rows > 0, rows <= header_l['nrows']-1)
+        rows = rows[ind_r]
+        ind_c = np.logical_and(cols > 0, cols <= header_l['ncols']-1)
+        cols = cols[ind_c]
+        array_small = self.array[ind_r, :]
+        array_small = array_small[:, ind_c]
+        rows_grid, cols_grid = np.meshgrid(rows, cols, indexing='ij')
+        obj_large.array[rows_grid, cols_grid] = array_small
+        return obj_large
 
     def to_points(self):
         """ Get X and Y coordinates of all raster cells
@@ -382,7 +455,7 @@ class Raster(object):
         """
         sp.arcgridwrite(output_file, self.array, self.header, compression)
         # if projection is defined, write .prj file for asc file
-        if export_prj & (self.crs is not None):
+        if export_prj & hasattr(self, 'crs'):
             prj_file = output_file[0:-4]+'.prj'
             wkt = self.crs.to_wkt()
             with open(prj_file, "w") as prj:        
